/*
 * Copyright 2019-2023 NXP
 * All rights reserved.
 *
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */
#include "fsl_smartdma_prv.h"
#include "fsl_smartdma.h"

/* Component ID definition, used by tools. */
#ifndef FSL_COMPONENT_ID
#define FSL_COMPONENT_ID "platform.drivers.lpc_smartdma"
#endif

/*******************************************************************************
 * Definitions
 ******************************************************************************/

typedef void (*smartdma_func_t)(void);

typedef struct
{
    __IO uint32_t SMARTDMA_BOOT;         /* 0x20 */
    __IO uint32_t SMARTDMA_CTRL;         /* 0x24 */
    __I uint32_t SMARTDMA_PC;            /* 0x28 */
    __I uint32_t SMARTDMA_SP;            /* 0x2C */
    __IO uint32_t SMARTDMA_BREAK_ADDR;   /* 0x30 */
    __IO uint32_t SMARTDMA_BREAK_VECT;   /* 0x34 */
    __IO uint32_t SMARTDMA_EMER_VECT;    /* 0x38 */
    __IO uint32_t SMARTDMA_EMER_SEL;     /* 0x3C */
    __IO uint32_t SMARTDMA_ARM2SMARTDMA; /* 0x40 */
    __IO uint32_t SMARTDMA_SMARTDMA2ARM; /* 0x44 */
    __IO uint32_t SMARTDMA_PENDTRAP;     /* 0x48 */
} SMARTDMA_Type;

#define SMARTDMA_BASE 0x40027020
#define SMARTDMA      ((volatile SMARTDMA_Type *)SMARTDMA_BASE)

/*******************************************************************************
 * Variables
 ******************************************************************************/
static smartdma_func_t *s_smartdmaApiTable;
static smartdma_callback_t s_smartdmaCallback;
static void *s_smartdmaCallbackParam;

const uint8_t s_smartdmaDisplayFirmware[] = {
	0x54, 0x00, 0x10, 0x24, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x12, 0x00, 0x00, 0x00, 0x00, 0x28, 0x04, 0x00, 0x00, 0x2C, 0x14, 0x10,
	0x04, 0x18, 0x04, 0x27, 0x01, 0x94, 0x05, 0x01, 0x01, 0x54, 0x05, 0x00,
	0x01, 0x9C, 0x05, 0x02, 0x08, 0xDC, 0x15, 0x00, 0x06, 0x90, 0xC5, 0x00,
	0x01, 0x10, 0x05, 0x00, 0x01, 0x00, 0x05, 0x00, 0x01, 0x04, 0x05, 0x01,
	0x08, 0x46, 0x04, 0x00, 0x35, 0xB2, 0x01, 0x20, 0x00, 0x48, 0x00, 0x00,
	0x08, 0x44, 0x14, 0x00, 0x06, 0x4C, 0x07, 0x02, 0x1A, 0xC0, 0x10, 0x00,
	0x12, 0x00, 0x00, 0x00, 0x01, 0x0C, 0x1C, 0x01, 0x1C, 0xB4, 0x00, 0x00,
	0x18, 0xEC, 0x02, 0x20, 0x05, 0x80, 0x36, 0x32, 0x08, 0x88, 0x14, 0x00,
	0x01, 0x8C, 0x05, 0x00, 0x02, 0xC0, 0x24, 0x00, 0x08, 0xCE, 0x51, 0x00,
	0x35, 0xB2, 0x01, 0x20, 0x06, 0x54, 0x15, 0x00, 0x01, 0x8C, 0x05, 0x01,
	0x02, 0xC0, 0x54, 0x00, 0x06, 0x10, 0x85, 0x00, 0x15, 0xFA, 0x00, 0x20,
	0x1C, 0xB4, 0x00, 0x00, 0x18, 0xEC, 0x02, 0x20, 0x04, 0x8C, 0x01, 0x32,
	0x0D, 0xCE, 0x14, 0x00, 0x35, 0xB2, 0x01, 0x20, 0x14, 0x00, 0x00, 0x00,
	0x00, 0x28, 0x04, 0x00, 0x00, 0x00, 0xBE, 0x0D, 0x00, 0x04, 0xDC, 0x06,
	0x00, 0x08, 0x66, 0x0B, 0x0E, 0x00, 0x10, 0x00, 0x0E, 0x2C, 0x20, 0x00,
	0x1C, 0xB4, 0x00, 0x00, 0x15, 0xB2, 0x01, 0x20
};

const uint32_t s_smartdmaDisplayFirmwareSize = sizeof(s_smartdmaDisplayFirmware);

/*******************************************************************************
 * Prototypes
 ******************************************************************************/

/*******************************************************************************
 * Codes
 ******************************************************************************/
/*!
 * brief Initialize the SMARTDMA.
 *
 * param apiMemAddr The address firmware will be copied to.
 * param firmware The firmware to use.
 * param firmwareSizeByte Size of firmware.
 */
void SMARTDMA_Init(uint32_t apiMemAddr, const void *firmware, uint32_t firmwareSizeByte)
{
    SMARTDMA_InitWithoutFirmware();
    SMARTDMA_InstallFirmware(apiMemAddr, firmware, firmwareSizeByte);
}

/*!
 * brief Initialize the SMARTDMA.
 *
 * This function is similar with SMARTDMA_Init, the difference is this function
 * does not install the firmware, the firmware could be installed using
 * SMARTDMA_InstallFirmware.
 */
void SMARTDMA_InitWithoutFirmware(void)
{
    /* Clear Smart DMA RAM */
    RESET_PeripheralReset(kSMART_DMA_RST_SHIFT_RSTn);
    CLOCK_EnableClock(kCLOCK_Smartdma);
}

/*!
 * @brief Install the firmware.
 *
 * param apiMemAddr The address firmware will be copied to.
 * param firmware The firmware to use.
 * param firmwareSizeByte Size of firmware.
 */
void SMARTDMA_InstallFirmware(uint32_t apiMemAddr, const void *firmware, uint32_t firmwareSizeByte)
{
    (void)memcpy((void *)(uint8_t *)apiMemAddr, firmware, firmwareSizeByte);
    SMARTDMA->SMARTDMA_CTRL = (0xC0DE0000U | (1U << SMARTDMA_ENABLE_GPISYNCH));
    s_smartdmaApiTable      = (smartdma_func_t *)apiMemAddr;
}

/*!
 * brief Install the complete callback function..
 *
 * param callback The callback called when smartdma program finished.
 */
void SMARTDMA_InstallCallback(smartdma_callback_t callback, void *param)
{
    s_smartdmaCallback      = callback;
    s_smartdmaCallbackParam = param;
}

/*!
 * brief Boot the SMARTDMA to run program.
 *
 * param apiIndex Index of the API to call.
 * param pParam Pointer to the parameter.
 * param mask Value set to SMARTDMA_ARM2SMARTDMA[0:1].
 */
void SMARTDMA_Boot(uint32_t apiIndex, void *pParam, uint8_t mask)
{
    SMARTDMA->SMARTDMA_ARM2SMARTDMA = (uint32_t)(uint8_t *)pParam | (uint32_t)mask;
    SMARTDMA->SMARTDMA_BOOT         = (uint32_t)(s_smartdmaApiTable[apiIndex]);
    SMARTDMA->SMARTDMA_CTRL = 0xC0DE0011U | (0U << SMARTDMA_MASK_RESP) | (0U << SMARTDMA_ENABLE_AHBBUF); /* BOOT */
};

/*!
 * brief Deinitialize the SMARTDMA.
 */
void SMARTDMA_Deinit(void)
{
    SMARTDMA->SMARTDMA_CTRL = 0xC0DE0000U;
    CLOCK_DisableClock(kCLOCK_Smartdma);
}

/*!
 * brief Reset the SMARTDMA.
 */
void SMARTDMA_Reset(void)
{
    RESET_PeripheralReset(kSMART_DMA_RST_SHIFT_RSTn);
    SMARTDMA->SMARTDMA_CTRL = (0xC0DE0000U | (1U << SMARTDMA_ENABLE_GPISYNCH));
}

/*!
 * brief SMARTDMA IRQ.
 */
void SMARTDMA_HandleIRQ(void)
{
    if (NULL != s_smartdmaCallback)
    {
        s_smartdmaCallback(s_smartdmaCallbackParam);
    }
}

void SDMA_IRQHandler(void);
void SDMA_IRQHandler(void)
{
    SMARTDMA_HandleIRQ();
}
